
# THIS FILE IS GENERATED FROM PYART SETUP.PY
short_version = '1.1.2'
version = '1.1.2'
full_version = '1.1.2.dev+c4b1c64'
git_revision = 'c4b1c64ac662da993d598303fc5d88fb7d8278f5'
release = False

if not release:
    version = full_version
